import os
import sys
import argparse
import difflib
import json
import logging
import pluggy
import colorama
import boto3
from pathlib import Path

from . lib.profile import aggregate_profiles
from . lib.config_management import load_config
from . lib.aws_files import get_aws_files
from . lib import exceptions
from . lib.logger import logger
from . lib.safe_print import safe_print
from . lib import constants
from . lib import saml as saml
from . lib import aws as aws_lib
from . import hookspec
from . import default_plugins


class Awsume(object):
    def __init__(self, is_interactive: bool = True):
        logger.debug('Initalizing app')
        self.plugin_manager = self.get_plugin_manager()
        self.config = load_config()
        self.config['is_interactive'] = is_interactive
        self.is_interactive = is_interactive
        colorama.init(autoreset=True)


    def get_plugin_manager(self) -> pluggy.PluginManager:
        logger.debug('Creating plugin manager')
        pm = pluggy.PluginManager('awsume')
        pm.add_hookspecs(hookspec)
        logger.debug('Loading plugins')
        pm.register(default_plugins)
        pm.load_setuptools_entrypoints('awsume')
        return pm


    def parse_args(self, system_arguments: list) -> argparse.Namespace:
        logger.debug('Gathering arguments')
        epilog = """Thank you for using AWSume! Check us out at https://trek10.com"""
        description="""Awsume - A cli that makes using AWS IAM credentials easy"""
        argument_parser = argparse.ArgumentParser(
            prog='awsume',
            description=description,
            epilog=epilog,
            formatter_class=lambda prog: (argparse.RawDescriptionHelpFormatter(prog, max_help_position=80, width=80)), # pragma: no cover
        )
        self.plugin_manager.hook.pre_add_arguments(
            config=self.config,
        )
        self.plugin_manager.hook.add_arguments(
            config=self.config,
            parser=argument_parser,
        )
        logger.debug('Parsing arguments')
        args = argument_parser.parse_args(system_arguments)
        logger.debug('Handling arguments')
        if args.refresh_autocomplete:
            autocomplete_file = Path('~/.awsume/autocomplete.json').expanduser()
            result = self.plugin_manager.hook.get_profile_names(
                config=self.config,
                arguments=args,
            )
            profile_names = [y for x in result for y in x]
            json.dump({'profile-names': profile_names}, open(autocomplete_file, 'w'))
            exit(0)
        if args.list_plugins:
            for plugin_name, _ in self.plugin_manager.list_name_plugin():
                if 'default_plugins' not in plugin_name:
                    safe_print(plugin_name, color=colorama.Fore.LIGHTCYAN_EX)
            exit(0)
        self.plugin_manager.hook.post_add_arguments(
            config=self.config,
            arguments=args,
            parser=argument_parser,
        )
        args.system_arguments = system_arguments
        return args


    def get_profiles(self, args: argparse.Namespace) -> dict:
        logger.debug('Gathering profiles')
        config_file, credentials_file = get_aws_files(args, self.config)
        self.plugin_manager.hook.pre_collect_aws_profiles(
            config=self.config,
            arguments=args,
            credentials_file=credentials_file,
            config_file=config_file,
        )
        aws_profiles_result = self.plugin_manager.hook.collect_aws_profiles(
            config=self.config,
            arguments=args,
            credentials_file=credentials_file,
            config_file=config_file,
        )
        profiles = aggregate_profiles(aws_profiles_result)
        self.plugin_manager.hook.post_collect_aws_profiles(
            config=self.config,
            arguments=args,
            profiles=profiles,
        )
        return profiles


    def get_saml_credentials(self, args: argparse.Namespace, profiles: dict) -> dict:
        assertion = self.plugin_manager.hook.get_credentials_with_saml(
            config=self.config,
            arguments=args,
        )
        assertion = next((_ for _ in assertion if _), None) # pragma: no cover
        if not assertion:
            safe_print('No assertion to use!', colorama.Fore.RED)
            if self.is_interactive:
                exit(1)
            raise exceptions.SAMLAssertionNotFoundError()
        try:
            roles = saml.parse_assertion(assertion)
        except exceptions.SAMLAssertionParseError:
            safe_print('Unable to parse SAML assertion', colorama.Fore.RED)
            if self.is_interactive:
                exit(1)
            raise
        if not roles:
            safe_print('No roles found in the saml assertion', colorama.Fore.RED)
            if self.is_interactive:
                exit(1)
            raise exceptions.SAMLAssertionMissingRoleError()
        role_arn = None
        principal_arn = None
        role_duration = args.role_duration or int(self.config.get('role-duration', '0'))

        if len(roles) > 1:
            if args.role_arn and args.principal_arn:
                principal_plus_role_arn = ','.join(args.role_arn, args.principal_arn)
                if self.config.get('fuzzy-match'):
                    choice = difflib.get_close_matches(principal_plus_role_arn, roles, cutoff=0)[0]
                    safe_print('Closest match: {}'.format(choice))
                else:
                    if principal_plus_role_arn not in roles:
                        raise exceptions.SAMLRoleNotFoundError(args.principal_arn, args.role_arn)
                    else:
                        choice = principal_plus_role_arn
            elif args.profile_name:
                profile_role_arn = profiles.get(args.profile_name, {}).get('role_arn')
                principal_arn = profiles.get(args.profile_name, {}).get('principal_arn')
                if profile_role_arn is None or principal_arn is None:
                    raise exceptions.InvalidProfileError(args.profile_name, 'both role_arn and principal_arn are necessary for saml profiles')
                principal_plus_profile_role_arn = ','.join([principal_arn, profile_role_arn])
                if principal_plus_profile_role_arn in roles:
                    choice = principal_plus_profile_role_arn
                else:
                    raise exceptions.SAMLRoleNotFoundError(principal_arn, profile_role_arn)
                safe_print('Match: {}'.format(choice))
            else:
                for index, choice in enumerate(roles):
                    safe_print('{}) {}'.format(index, choice), color=colorama.Fore.LIGHTYELLOW_EX)
                safe_print('Which role do you want to assume? > ', end='', color=colorama.Fore.LIGHTCYAN_EX)
                response = input()
                if response.isnumeric():
                    choice = roles[int(response)]
                else:
                    choice = difflib.get_close_matches(response, roles, cutoff=0)[0]
            role_arn = choice.split(',')[1]
            principal_arn = choice.split(',')[0]
        else:
            role_arn = roles[0].split(',')[1]
            principal_arn = roles[0].split(',')[0]
        safe_print('Assuming role: {},{}'.format(principal_arn, role_arn), color=colorama.Fore.GREEN)
        credentials = aws_lib.assume_role_with_saml(
            role_arn,
            principal_arn,
            assertion,
            region=None,
            role_duration=role_duration,
        )
        return credentials


    def get_credentials(self, args: argparse.Namespace, profiles: dict) -> dict:
        logger.debug('Getting credentials')
        self.plugin_manager.hook.pre_get_credentials(
            config=self.config,
            arguments=args,
            profiles=profiles,
        )
        try:
            if args.json or not sys.stdin.isatty(): # piping/sending credentials to awsume directly
                logger.debug('Pulling credentials from {}'.format('json parameter' if args.json else 'stdin'))
                args.target_profile_name = 'json' if args.json else 'stdin'
                json_input = args.json if args.json else sys.stdin.read()
                try:
                    credentials = json.loads(json_input)
                    if 'Credentials' in credentials:
                        credentials = credentials['Credentials']
                except json.JSONDecodeError:
                    safe_print('Data is not valid json')
                    if self.is_interactive:
                        exit(1)
                    raise
                credentials = [credentials]
            elif args.with_saml:
                logger.debug('Pulling credentials from saml')
                credentials = self.get_saml_credentials(args, profiles)
                credentials = [credentials]
            elif args.with_web_identity:
                logger.debug('Pulling credentials from web identity')
                credentials = self.plugin_manager.hook.get_credentials_with_web_identity(
                    config=self.config,
                    arguments=args,
                )
            else:
                logger.debug('Pulling credentials from default awsume flow')
                credentials = self.plugin_manager.hook.get_credentials(config=self.config, arguments=args, profiles=profiles)
        except exceptions.ProfileNotFoundError as e:
            safe_print(e, colorama.Fore.RED)
            logger.debug('', exc_info=True)
            self.plugin_manager.hook.catch_profile_not_found_exception(config=self.config, arguments=args, profiles=profiles, error=e)
            if self.is_interactive:
                exit(1)
            raise
        except exceptions.InvalidProfileError as e:
            safe_print(e, colorama.Fore.RED)
            logger.debug('', exc_info=True)
            self.plugin_manager.hook.catch_invalid_profile_exception(config=self.config, arguments=args, profiles=profiles, error=e)
            if self.is_interactive:
                exit(1)
            raise
        except exceptions.UserAuthenticationError as e:
            safe_print(e, colorama.Fore.RED)
            logger.debug('', exc_info=True)
            self.plugin_manager.hook.catch_user_authentication_error(config=self.config, arguments=args, profiles=profiles, error=e)
            if self.is_interactive:
                exit(1)
            raise
        except exceptions.RoleAuthenticationError as e:
            safe_print(e, colorama.Fore.RED)
            logger.debug('', exc_info=True)
            self.plugin_manager.hook.catch_role_authentication_error(config=self.config, arguments=args, profiles=profiles, error=e)
            if self.is_interactive:
                exit(1)
            raise
        credentials = next((_ for _ in credentials if _), {}) # pragma: no cover
        self.plugin_manager.hook.post_get_credentials(
            config=self.config,
            arguments=args,
            profiles=profiles,
            credentials=credentials,
        )
        if not credentials:
            safe_print('No credentials to awsume', colorama.Fore.RED)
            if self.is_interactive:
                exit(1)
            raise exceptions.NoCredentialsError()
        return credentials


    def export_data(self, credentials: dict, awsume_flag: str, awsume_list: list):
        logger.debug('Exporting data')
        if self.is_interactive:
            print(awsume_flag, end=' ')
            print(' '.join(awsume_list))
        return boto3.Session(
            aws_access_key_id=credentials.get('AccessKeyId'),
            aws_secret_access_key=credentials.get('SecretAccessKey'),
            aws_session_token=credentials.get('SessionToken'),
            region_name=credentials.get('Region'),
        )


    def run(self, system_arguments: list):
        args = self.parse_args(system_arguments)
        profiles = self.get_profiles(args)
        credentials = self.get_credentials(args, profiles)

        if args.auto_refresh:
            return self.export_data(credentials, 'Auto', [
                'autoawsume-{}'.format(args.target_profile_name),
                credentials.get('Region'),
                args.target_profile_name,
            ])
        else:
            return self.export_data(credentials, 'Awsume', [
                str(credentials.get('AccessKeyId')),
                str(credentials.get('SecretAccessKey')),
                str(credentials.get('SessionToken')),
                str(credentials.get('Region')),
                str(args.target_profile_name),
            ])



import argparse

from . import aws_files as aws_files_lib
from . import profile as profile_lib


def create_autoawsume_profile(config: dict, arguments: argparse.Namespace, role_session: dict, source_session: dict):
    _, credentials_file = aws_files_lib.get_aws_files(arguments, config)
    autoawsume_profile_name = 'autoawsume-{}'.format(arguments.target_profile_name)
    profile = profile_lib.credentials_to_profile(role_session)
    profile['expiration'] = role_session.get('Expiration').strftime('%Y-%m-%d %H:%M:%S')
    profile['source_expiration'] = source_session.get('Expiration').strftime('%Y-%m-%d %H:%M:%S')
    profile['awsumepy_command'] = ' '.join(arguments.system_arguments)
    aws_files_lib.add_section(autoawsume_profile_name, profile, credentials_file, True)



from awsume import __data__

__VERSION__ = __data__.version
__NAME__ = __data__.name
__AUTHOR__ = __data__.author
__AUTHOR_EMAIL__ = __data__.author_email
__DESCRIPTION__ = __data__.description
__LICENSE__ = __data__.license
__HOMEPAGE__ = __data__.homepage
__MESSAGE__ = __data__.message



import sys
import logging

logger = logging.getLogger('awsume') # type: logging.Logger
LOG_HANDLER = logging.StreamHandler()
LOG_HANDLER.setFormatter(logging.Formatter('[%(asctime)s] %(filename)s:%(funcName)s : [%(levelname)s] %(message)s'))
logger.addHandler(LOG_HANDLER)





import argparse, os, sys
from . import alias, autocomplete


def parse_args(argv: sys.argv) -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument('--shell',
        dest='shell',
        metavar='shell',
        help='The shell you will use awsume under',
        required=True,
        choices=['bash', 'zsh', 'powershell']
    )
    parser.add_argument('--autocomplete-file',
        dest='autocomplete_file',
        metavar='autocomplete_file',
        required=True,
        help='The file you want the autocomplete script to be defined in',
    )
    parser.add_argument('--alias-file',
        default=None,
        dest='alias_file',
        metavar='alias_file',
        required=False,
        help='The file you want the alias to be defined in',
    )
    args = parser.parse_args(argv)

    if args.shell in ['powershell'] and args.alias_file:
        parser.error('No alias file is needed for shell: powershell')

    return args


def run(shell: str, alias_file: str, autocomplete_file: str):
    if alias_file:
        alias.main(shell, alias_file)
    if autocomplete_file:
        autocomplete.main(shell, autocomplete_file)


def main():
    args = parse_args(sys.argv[1:])
    run(args.shell, args.alias_file, args.autocomplete_file)



import os, pathlib
from distutils.spawn import find_executable

DEFAULT_ALIAS = 'alias awsume=". awsume"'
PYENV_ALIAS = r'alias awsume=". \$(pyenv which awsume)"'

def main(shell: str, alias_file: str):
    alias_file = str(pathlib.Path(alias_file).expanduser())
    alias = PYENV_ALIAS if find_executable('pyenv') and not find_executable('pipx') else DEFAULT_ALIAS

    basedir = os.path.dirname(alias_file)
    if basedir and not os.path.exists(basedir):
        os.makedirs(basedir)
    open(alias_file, 'a').close()

    if alias in open(alias_file, 'r').read():
        print('Alias already in ' + alias_file)
    else:
        with open(alias_file, 'a') as f:
            f.write('\n#AWSume alias to source the AWSume script\n')
            f.write(alias)
            f.write('\n')
        print('Wrote alias to ' + alias_file)



import argparse
import dateutil
import boto3
import colorama
from datetime import datetime

from . import profile as profile_lib
from . import cache as cache_lib
from . exceptions import RoleAuthenticationError, UserAuthenticationError
from . logger import logger
from . safe_print import safe_print


DEFAULT_REGION = 'us-east-1'


def assume_role(
    source_credentials: dict,
    role_arn: str,
    session_name: str,
    external_id: str = None,
    region: str = None,
    role_duration: int = None,
    mfa_serial: str = None,
    mfa_token: str = None,
) -> dict:
    if len(session_name) < 2:
        session_name = session_name.center(2, '_')

    logger.debug('Assuming role: {}'.format(role_arn))
    logger.debug('Session name: {}'.format(session_name))
    role_sts_client = boto3.session.Session(
        aws_access_key_id=source_credentials.get('AccessKeyId'),
        aws_secret_access_key=source_credentials.get('SecretAccessKey'),
        aws_session_token=source_credentials.get('SessionToken'),
        region_name=region or DEFAULT_REGION,
    ).client('sts') # type: botostubs.STS

    try:
        kwargs = { 'RoleSessionName': session_name, 'RoleArn': role_arn }
        if external_id:
            kwargs['ExternalId'] = external_id
        if role_duration:
            kwargs['DurationSeconds'] = int(role_duration)
        if mfa_serial:
            kwargs['SerialNumber'] = mfa_serial
            kwargs['TokenCode'] = mfa_token or profile_lib.get_mfa_token()
        role_session = role_sts_client.assume_role(**kwargs).get('Credentials')
        role_session['Expiration'] = role_session['Expiration'].astimezone(dateutil.tz.tzlocal())
        role_session['Region'] = region or DEFAULT_REGION
    except Exception as e:
        raise RoleAuthenticationError(str(e))
    logger.debug('Role credentials received')
    return role_session


def get_session_token(
    source_credentials: dict,
    region: str = None,
    mfa_serial: str = None,
    mfa_token: str = None,
    ignore_cache: bool = False,
) -> dict:
    cache_file_name = 'aws-credentials-' + source_credentials.get('AccessKeyId')
    cache_session = cache_lib.read_aws_cache(cache_file_name)

    if cache_lib.valid_cache_session(cache_session) and not ignore_cache:
        logger.debug('Using cache session')
        if region:
            cache_session['Region'] = region
        user_session = cache_session
    else:
        logger.debug('Getting session token')
        user_sts_client = boto3.session.Session(
            aws_access_key_id=source_credentials.get('AccessKeyId'),
            aws_secret_access_key=source_credentials.get('SecretAccessKey'),
            aws_session_token=source_credentials.get('SessionToken'),
            region_name=region or DEFAULT_REGION,
        ).client('sts') # type: botostubs.STS
        try:
            user_session = user_sts_client.get_session_token(
                SerialNumber=mfa_serial if mfa_serial else None,
                TokenCode=None if not mfa_serial else (mfa_token or profile_lib.get_mfa_token()),
            ).get('Credentials')
            user_session['Expiration'] = user_session['Expiration'].astimezone(dateutil.tz.tzlocal())
            user_session['Region'] = region or DEFAULT_REGION
        except Exception as e:
            raise UserAuthenticationError(str(e))
        logger.debug('Session token received')
        cache_lib.write_aws_cache(cache_file_name, user_session)
    return user_session


def get_account_id(credentials: dict):
    try:
        sts_client = boto3.session.Session(
            aws_access_key_id=credentials.get('AccessKeyId'),
            aws_secret_access_key=credentials.get('SecretAccessKey'),
            aws_session_token=credentials.get('SessionToken'),
            region_name=credentials.get('Region', 'us-east-1'),
        ).client('sts') # type: botostubs.STS
        response = sts_client.get_caller_identity()
        return response.get('Account', 'Unavailable')
    except:
        return 'Unavailable'


def assume_role_with_saml(
    role_arn: str,
    principal_arn: str,
    saml_assertion: str,
    region: str = 'us-east-1',
    role_duration: int = None,
) -> dict:
    logger.debug('Assuming role with saml: {}'.format(role_arn))
    role_sts_client = boto3.session.Session().client('sts') # type: botostubs.STS

    try:
        kwargs = { 'RoleArn': role_arn, 'PrincipalArn': principal_arn, 'SAMLAssertion': saml_assertion }
        if role_duration:
            kwargs['DurationSeconds'] = int(role_duration)
        role_session = role_sts_client.assume_role_with_saml(**kwargs).get('Credentials')
        role_session['Expiration'] = role_session['Expiration'].astimezone(dateutil.tz.tzlocal())
        role_session['Region'] = region
    except Exception as e:
        raise RoleAuthenticationError(str(e))
    logger.debug('Role credentials received')
    safe_print('Role credentials will expire {}'.format(profile_lib.parse_time(role_session['Expiration'])), colorama.Fore.GREEN)
    return role_session



class ProfileNotFoundError(Exception):
    """"""
    def __init__(self, profile_name='', message=''):
        self.profile_name = profile_name
        self.message = message
    def __str__(self):
        if self.message:
            return self.message
        return 'Profile {} not found.'.format(self.profile_name)


class InvalidProfileError(Exception):
    """"""
    def __init__(self, profile_name, message=''):
        self.profile_name = profile_name
        self.message = message
    def __str__(self):
        return 'Invalid profile {} {}'.format(self.profile_name, self.message)


class UserAuthenticationError(Exception):
    """"""
    def __init__(self, message=''):
        self.message = message
    def __str__(self):
        return self.message if self.message else 'Unable to get session token'


class RoleAuthenticationError(Exception):
    """"""
    def __init__(self, message=''):
        self.message = message
    def __str__(self):
        return self.message if self.message else 'Unable to assume role'


class SAMLAssertionNotFoundError(Exception):
    """"""
    def __init__(self, message=''):
        self.message = message
    def __str__(self):
        return self.message if self.message else 'No SAML assertion'


class SAMLAssertionMissingRoleError(Exception):
    """"""
    def __init__(self, message=''):
        self.message = message
    def __str__(self):
        return self.message if self.message else 'No role in the SAML assertion'


class SAMLRoleNotFoundError(Exception):
    """"""
    def __init__(self, principal_arn, role_arn, message=''):
        self.role_arn = role_arn
        self.principal_arn = principal_arn
        self.message = message
    def __str__(self):
        return self.message if self.message else 'No match for SAML principal and role: {},{}'.format(self.principal_arn, self.role_arn)


class SAMLAssertionParseError(Exception):
    """"""
    def __init__(self, message=''):
        self.message = message
    def __str__(self):
        return self.message if self.message else 'Cannot parse SAML assertion'


class NoCredentialsError(Exception):
    """"""
    def __init__(self, message=''):
        self.message = message
    def __str__(self):
        return self.message if self.message else 'No credentials'



import os
import sys
import json
import configparser
from pathlib import Path


def get_aws_files() -> tuple:
    config_file = os.environ.get('AWS_CONFIG_FILE') if os.environ.get('AWS_CONFIG_FILE') else '~/.aws/config'
    credentials_file = os.environ.get('AWS_CREDENTIALS_FILE') if os.environ.get('AWS_CREDENTIALS_FILE') else '~/.aws/credentials'
    return str(Path(config_file).expanduser()), str(Path(credentials_file).expanduser())


def get_profile_names(credentials_file: str, config_file: str) -> list:
    credentials = configparser.ConfigParser()
    credentials.read(credentials_file)
    profiles = list(credentials._sections.keys())

    config = configparser.ConfigParser()
    config.read(config_file)
    config_profiles = [_.replace('profile ', '') for _ in list(config._sections.keys())]

    return uniquely_concat_lists(profiles, config_profiles)


def uniquely_concat_lists(list1, list2):
    for element in list2:
        if element not in list1:
            list1.append(element)
    return list1


def main():
    config, credentials = get_aws_files()
    profile_names = get_profile_names(credentials, config)
    autocomplete_file = str(Path('~/.awsume/autocomplete.json').expanduser())
    if os.path.isfile(autocomplete_file):
        autocomplete = json.load(open(autocomplete_file))
        profile_names = uniquely_concat_lists(profile_names, autocomplete['profile-names'])
    print('\n'.join(profile_names))


if __name__ == "__main__":
    main()



import os
import argparse
import configparser
import colorama
from pathlib import Path

from . import constants
from . safe_print import safe_print


def get_aws_files(args: argparse.Namespace, config: dict) -> tuple:
    if os.environ.get('AWS_CONFIG_FILE'):
        config_file = os.environ.get('AWS_CONFIG_FILE')
    elif args and args.config_file:
        config_file = args.config_file
    elif config and config.get('config-file'):
        config_file = config.get('config-file')
    else:
        config_file = constants.DEFAULT_CONFIG_FILE

    if os.environ.get('AWS_SHARED_CREDENTIALS_FILE'):
        credentials_file = os.environ.get('AWS_SHARED_CREDENTIALS_FILE')
    elif args and  args.credentials_file:
        credentials_file = args.credentials_file
    elif config and config.get('credentials-file'):
        credentials_file = config.get('credentials-file')
    else:
        credentials_file = constants.DEFAULT_CREDENTIALS_FILE

    return str(Path(config_file)), str(Path(credentials_file))


def add_section(name: str, section: dict, file_name: str, overwrite: bool = False):
    config = configparser.ConfigParser()
    config.read(file_name)
    if config.has_section(name):
        if not overwrite:
            safe_print('Cannot overwrite data in {}'.format(file_name), colorama.Fore.RED)
            return
        config.remove_section(name)
    config.add_section(name)
    for key in section:
        config.set(name, key, str(section[key]))
    config.write(open(str(file_name), 'w'))


def delete_section(name: str, file_name: str):
    config = configparser.ConfigParser()
    config.read(file_name)
    if config.has_section(name):
        config.remove_section(name)
    config.write(open(str(file_name), 'w'))


def read_aws_file(file_name: str) -> dict:
    config = configparser.ConfigParser()
    config.read(file_name)
    profiles = {k: dict(v) for k, v in config._sections.items()}
    return profiles



import os
import json
import dateutil
from datetime import datetime

from . import constants
from . logger import logger


def ensure_cache_dir():
    cache_dir = str(constants.AWSUME_CACHE_DIR)
    if not os.path.exists(cache_dir):
        os.makedirs(cache_dir)


def read_aws_cache(cache_file_name: str) -> dict:
    ensure_cache_dir()
    cache_path = str(constants.AWSUME_CACHE_DIR) + '/' + cache_file_name
    logger.debug('Cache file path: ' + cache_path)
    session = {}
    if os.path.isfile(cache_path):
        logger.debug('Cache file found')
        try:
            session = json.load(open(cache_path))
            if session.get('Expiration') and type(session.get('Expiration')) == str:
                session['Expiration'] = datetime.strptime(session['Expiration'], '%Y-%m-%d %H:%M:%S')
        except:
            logger.debug('There was an error reading from the cache file', exc_info=True)
    return session


def write_aws_cache(cache_file_name: str, session: dict) -> dict:
    ensure_cache_dir()
    cache_path = str(constants.AWSUME_CACHE_DIR) + '/' + cache_file_name
    logger.debug('Cache file path: ' + cache_path)
    expiration = session['Expiration'].astimezone(dateutil.tz.tzlocal())
    expiration = expiration.strftime('%Y-%m-%d %H:%M:%S')
    try:
        json.dump({
            **session,
            'Expiration': expiration,
        }, open(cache_path, 'w'), indent=2, default=str)
    except:
        logger.debug('There was an error writing to the cache file', exc_info=True)
    session['Expiration'] = datetime.strptime(expiration, '%Y-%m-%d %H:%M:%S')
    return session


def valid_cache_session(cache_session: dict) -> bool:
    if cache_session.get('Expiration'):
        session_expiration = cache_session['Expiration']
        if type(cache_session['Expiration']) == str:
            session_expiration = datetime.strptime(session_expiration, '%Y-%m-%d %H:%M:%S')
        if session_expiration <= datetime.now():
            logger.debug('Cache session has expired')
            return False
    if 'AccessKeyId' not in cache_session:
        return False
    if 'SecretAccessKey' not in cache_session:
        return False
    if 'SessionToken' not in cache_session:
        return False
    logger.debug('Cache session is valid')
    return True



import json
import subprocess
import configparser
import time
from datetime import datetime, timedelta

from ..awsumepy.lib.aws_files import get_aws_files, delete_section


def main():
    _, credentials_file = get_aws_files(None, None)
    while True:
        credentials = configparser.ConfigParser()
        credentials.read(credentials_file)
        auto_profiles = {k: dict(v) for k, v in credentials._sections.items() if k.startswith('autoawsume-')}

        expirations = []
        for _, auto_profile in auto_profiles.items():
            expiration = datetime.strptime(auto_profile['expiration'], '%Y-%m-%d %H:%M:%S')
            source_expiration = datetime.strptime(auto_profile['source_expiration'], '%Y-%m-%d %H:%M:%S')

            if source_expiration < datetime.now() + timedelta(minutes=5):
                continue

            if expiration < datetime.now() + timedelta(minutes=5):
                subprocess.run(auto_profile.get('awsumepy_command').split(' '), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                expirations.append(datetime.now() + timedelta(hours=1))
            else:
                expirations.append(expiration)

        if not expirations:
            break

        earliest_expiration = min(expirations)
        time_to_sleep = (earliest_expiration - datetime.now().replace(tzinfo=earliest_expiration.tzinfo)).total_seconds()
        time.sleep(time_to_sleep)



import argparse
import psutil

from ..awsumepy.lib.aws_files import delete_section, get_aws_files, read_aws_file
from ..awsumepy.lib.logger import logger

def kill_autoawsume():
    logger.debug('Killing autoawsume')
    for proc in psutil.process_iter():
        try:
            for command_string in proc.cmdline():
                if 'autoawsume' in command_string:
                    proc.kill()
        except Exception:
            pass


def kill(arguments: argparse.Namespace):
    _, credentials_file = get_aws_files(None, None)
    if arguments.profile_name:
        logger.debug('Stoping auto-refresh of profile {}'.format(arguments.profile_name))
        delete_section('autoawsume-{}'.format(arguments.profile_name), credentials_file)
        profiles = read_aws_file(credentials_file)
        profile_names = [_ for _ in profiles]
        if any(['autoawsume-' in _ for _ in profile_names]):
            print('Stop {}'.format(arguments.profile_name))
            return
        else:
            logger.debug('There were not more autoawsume profiles, stopping autoawsume')
            print('Kill')
            kill_autoawsume()
    else:
        logger.debug('Stopping all auto refreshing and removing autoawsume profiles')
        kill_autoawsume()
        profiles = read_aws_file(credentials_file)
        for profile in profiles:
            if 'autoawsume-' in profile:
                delete_section(profile, credentials_file)
        print('Kill')



import json
import os
import sys

import colorama

import yaml

from . import constants
from .logger import logger
from .safe_print import safe_print


defaults = {
    'role-duration': 0,
    'colors': True,
    'fuzzy-match': False,
}


def load_config() -> dict:
    if not os.path.exists(str(constants.AWSUME_DIR)):
        os.makedirs(str(constants.AWSUME_DIR))
    if not os.path.isfile(str(constants.AWSUME_CONFIG)):
        open(str(constants.AWSUME_CONFIG), 'a').close()

    options = None
    try:
        options = yaml.safe_load(open(str(constants.AWSUME_CONFIG), 'r'))
    except Exception:
        safe_print('Cannot parse config file: {}'.format(constants.AWSUME_CONFIG), colorama.Fore.RED)
        exit(1)
    if options is None:
        options = defaults
        write_config(options)
    return options


def write_config(config: dict):
    if not os.path.exists(str(constants.AWSUME_DIR)):
        os.makedirs(str(constants.AWSUME_DIR))
    if not os.path.isfile(str(constants.AWSUME_CONFIG)):
        open(str(constants.AWSUME_CONFIG), 'a').close()

    try:
        yaml.safe_dump(config, open(str(constants.AWSUME_CONFIG), 'w'), width=1000)
    except Exception as e:
        safe_print('Unable to write config: {}'.format(e), colorama.Fore.RED)


def handle_config(operations: list):
    logger.debug('Updating config: {}'.format(', '.join(operations)))
    config = load_config()

    if operations[0].lower() == 'get':
        if len(operations) != 2:
            safe_print('Must supply value to get')
            exit(1)
        logger.debug('Getting {}'.format(operations[1]))
        value = get_dict_parts(config, operations[1])
        safe_print(json.dumps(value))
        exit(0)

    if operations[0].lower() == 'list':
        if len(operations) != 1:
            safe_print('No operands are valid for operation "list"')
            exit(1)
        logger.debug('Listing config')
        yaml.safe_dump(config, sys.stderr, width=1000)
        exit(0)

    if operations[0].lower() == 'set':
        if len(operations) < 3:
            safe_print('Must supply value to set {} to'.format(operations[1]))
            exit(1)
        logger.debug('Setting {} to {}'.format(operations[1], operations[2]))
        value = get_value_from_args(operations[2:])
        config = update_dict_parts(config, operations[1], value)

    if operations[0].lower() in ['reset']:
        default_value = get_dict_parts(defaults, operations[1])
        if default_value is None:
            safe_print('Key does not have a default: {}'.format(operations[1]), colorama.Fore.YELLOW)
            exit(1)
        config = update_dict_parts(config, operations[1], default_value)
        safe_print('Reset key {} to {}'.format(operations[1], default_value), colorama.Fore.YELLOW)

    if operations[0].lower() in ['clear']:
        config, deleted = delete_dict_value_parts(config, operations[1])
        if deleted:
            safe_print('Deleted key {}'.format(operations[1]), colorama.Fore.YELLOW)

    write_config(config)


def get_value_from_args(args):
    values = args
    for index, value in enumerate(args):
        try:
            values[index] = json.loads(value)
        except json.JSONDecodeError:
            logger.debug('Cannot parse input', exc_info=True)
    return values[0] if len(values) == 1 else values


def get_dict_parts(obj, parts_string):
    parts = parts_string.split('.')
    location = obj
    for path in parts:
        if path in location:
            location = location[path]
        else:
            return None
    return location


def update_dict_parts(obj, parts_string, value):
    parts = parts_string.split('.')
    location = obj
    for path in parts[:-1]:
        if path in location:
            location = location[path]
        else:
            location[path] = {}
            location = location[path]
    location[parts[-1]] = value
    return obj


def delete_dict_value_parts(obj, parts_string):
    parts = parts_string.split('.')
    location = obj
    for path in parts[:-1]:
        if path in location:
            location = location[path]
        else:
            return obj, False
    del location[parts[-1]]
    return obj, True
